from typing import List
from lxml import etree
import logging

from .auth import Auth
from .relay import Relay

logger = logging.getLogger(__name__)


class KMTronicHubAPI:
    """Class to communicate with the KMTronic Web based API."""

    def __init__(self, auth: Auth):
        """Initialize the API and store the auth so we can make requests."""
        self.auth = auth
        self.host = auth.host
        self.relays = []

    @property
    def name(self)-> str:
        return self.host

    async def async_get_status(self) -> List[tuple]:
        """Return status on relays."""
        resp = await self.auth.request("status.xml")
        resp.raise_for_status()

        response_xml = etree.fromstring(await resp.text())
        response = response_xml.xpath("/response/*")

        status = {}
        for relay in response[1:]:
            relay_num = int(relay.tag[5:])
            relay_status = relay.text == "1"

            status[relay_num] = relay_status
        return status

    async def async_get_relays(self) -> List[Relay]:
        """Create relay representations from status information."""
        status = await self.async_get_status()
        self.relays = [
            Relay(number, status, self.auth) for number, status in status.items()
        ]

        return self.relays

    async def async_update_relays(self):
        """Update the status of each relay."""
        status = await self.async_get_status()

        logger.debug("Status of relays: %s", status)
        for relay in self.relays:
            relay.is_on = status[relay.id]



from aiohttp import ClientSession, ClientResponse, BasicAuth


class Auth:
    """Class to make authenticated requests."""

    def __init__(self, websession: ClientSession, host: str, user: str, password: str):
        """Initialize the auth."""
        self.websession = websession
        self.host = host
        self._auth = BasicAuth(user, password)

    async def request(self, path: str) -> ClientResponse:
        """Make a request."""

        return await self.websession.request(
            "GET",
            f"{self.host}/{path}",
            auth=self._auth,
        )



from pykmtronic.auth import Auth
import logging

logger = logging.getLogger(__name__)


class Relay:
    def __init__(self, number: int, status: bool, auth: Auth):
        self._relay = number
        self.auth = auth
        self.is_on = status  # True means ON

    @property
    def id(self) -> int:
        return self._relay

    @property
    def is_on(self):
        return self._is_on

    @is_on.setter
    def is_on(self, b):
        logger.debug(f"Relay{self._relay} is now {'ON' if b else 'OFF'}")
        self._is_on = b

    async def turn_on(self):
        logger.debug(f"Sending ... FF{self._relay:02}01")
        resp = await self.auth.request(f"FF{self._relay:02}01")
        resp.raise_for_status()
        self.is_on = True

    async def turn_off(self):
        logger.debug(f"Sending ... FF{self._relay:02}00")
        resp = await self.auth.request(f"FF{self._relay:02}00")
        resp.raise_for_status()
        self.is_on = False
